@@ -944,6 +944,30 @@ MacroAssembler::branchTestSymbolImpl(Condition cond, const T& t, Label* label)
     j(cond, label);
 }
 
+void MacroAssembler::branchTestBigInt(Condition cond, Register tag, Label* label) {
+    branchTestBigIntImpl(cond, tag, label);
+}
+
+void MacroAssembler::branchTestBigInt(Condition cond, const BaseIndex& address, Label* label) {
+    branchTestBigIntImpl(cond, address, label);
+}
+
+void MacroAssembler::branchTestBigInt(Condition cond, const ValueOperand& value, Label* label) {
+    branchTestBigIntImpl(cond, value, label);
+}
+
+template <typename T>
+void MacroAssembler::branchTestBigIntImpl(Condition cond, const T& t, Label* label) {
+    cond = testBigInt(cond, t);
+    j(cond, label);
+}
+
+void MacroAssembler::branchTestBigIntTruthy(bool truthy, const ValueOperand& value, Label* label)
+{
+    Condition cond = testBigIntTruthy(truthy, value);
+    j(cond, label);
+}
+
 void
 MacroAssembler::branchTestNull(Condition cond, Register tag, Label* label)
 {
